Package: httr
Title: Tools for Working with URLs and HTTP
Version: 1.4.8
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Useful tools for working with HTTP organised by HTTP verbs
    (GET(), POST(), etc). Configuration functions make it easy to control
    additional request components (authenticate(), add_headers() and so
    on).
License: MIT + file LICENSE
URL: https://httr.r-lib.org/, https://github.com/r-lib/httr
BugReports: https://github.com/r-lib/httr/issues
Depends: R (>= 3.6)
Imports: curl (>= 5.1.0), jsonlite, mime, openssl (>= 0.8), R6
Suggests: covr, httpuv, jpeg, knitr, png, readr, rmarkdown, testthat
        (>= 0.8.0), xml2
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-13 13:25:45 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-02-13 16:20:15 UTC
Built: R 4.6.0; ; 2026-04-26 05:16:18 UTC; unix
