% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response-status.r
\name{status_code}
\alias{status_code}
\title{Extract status code from response.}
\usage{
status_code(x)
}
\arguments{
\item{x}{A response}
}
\description{
Extract status code from response.
}
