% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doctor.R
\name{httr_dr}
\alias{httr_dr}
\title{Diagnose common configuration problems}
\usage{
httr_dr()
}
\description{
Currently one check: that curl uses nss.
}
